/*
   There are two important kinds of loop "patterns",
   counting-loops and open-ended-loops.

   With a counting-loop, we know exactly how many times
   the loop will iterate.

   With open-ended-loops, we do not know how many times
   the loop will iterate, and it may iterate for ever.
*/
import java.util.Scanner;
import java.util.Random;

public class Counting_vs_OpenEnded_ForLoops
{
   public static void main( String[] args )
   {
      // Here is a counting for-loop.
      // We know how many times it will loop.
      int start_value = 1;
      int end_value = 10;

      for (int i = start_value; i <= end_value; i++)  // try < instead of <=
      {
         System.out.println("i = " + i);
      }
      System.out.println();



      // Here is an open-ended for-loop.
      // We do not know how many times it will loop.
      Scanner sc = new Scanner(System.in);
      for (int input = 0; input >= 0; input = sc.nextInt())
      {
         System.out.print( "Please enter a negative number: " );
      }
      System.out.println("Thank you.\n");



      // Here is a for-loop that is neither a counting loop nor an open ended loop.
      // We do not know how many times it will loop, but it cannot loop for ever.
      Random random = new Random(); // random number generator
      for (int i = start_value; i <= end_value; i += 1 + random.nextInt(3))
      {
         System.out.println("i = " + i);
      }
      System.out.println();


      // Another for-loop that is neither a counting loop nor an open ended loop.
      for (int i = start_value; i <= end_value; /* empty */)
      {
         System.out.println("i = " + i);
         if (Math.random() < 0.5)  // "flip a coin"
         {
            i++;
         }
      }
      System.out.println();
   }
}
