/*
   There are two important kinds of loop "patterns",
   counting-loops and open-ended-loops.

   With a counting-loop, we know exactly how many times
   the loop will iterate.

   With open-ended-loops, we do not know how many times
   the loop will iterate, and it may iterate for ever.
*/
import java.util.Scanner;
import java.util.Random;

public class Counting_vs_OpenEnded_WhileLoops
{
   public static void main( String[] args )
   {
      // Here is a counting while-loop.
      // We know how many times it will loop.
      int start_value = 1;
      int end_value = 10;

      int i = start_value;
      while (i <= end_value)  // try < instead of <=
      {
         System.out.println("i = " + i);
         i++;
      }
      System.out.println();



      // Here is an open-ended while-loop.
      // We do not know how many times it will loop.
      Scanner sc = new Scanner(System.in);
      int input = 0;
      while (input >= 0)
      {
         System.out.print( "Please enter a negative number: " );
         input = sc.nextInt();
      }
      System.out.println("Thank you.\n");



      // Here is a while-loop that is neither a counting loop nor an open ended loop.
      // We do not know how many times it will loop, but it cannot loop for ever.
      i = start_value;
      Random random = new Random(); // random number generator
      while (i <= end_value)
      {
         System.out.println("i = " + i);
         i = i + 1 + random.nextInt(3); // randomly add 0, 1, or 2
      }
      System.out.println();


      // Another while-loop that is neither a counting loop nor an open ended loop.
      i = start_value;
      while (i <= end_value)
      {
         System.out.println("i = " + i);
         if (Math.random() < 0.5)  // "flip a coin"
         {
            i++;
         }
      }
      System.out.println();
   }
}
